/*                                                          */
/*         ... ... ... <<< dc_test.c >>> ... ... ...        */
/*                                                          */
/*              A sample interrupt handler of               */
/*                                                          */
/*                      A device driver                     */
/*          for Toyo CC/7x00 CAMAC crate controller         */
/*                on Linux 2.0 & 2.2 kernels                */
/*                                                          */

/* station number of modules */
#define TDC1      6
#define OUTREG    4
/* information of LAM source */
#define LAM_N     TDC1
#define LAM_A     0
/* some parameters */
#define MAX_EVLEN 1024

#ifdef OUTREG
static unsigned short  dc_outreg=0x0000;
#endif

/*
 *  Subsidiary function called at enabling interrupt
 */
static void dc_int_on() {
  int stat;
# if (defined(LAM_N) && defined(LAM_A))
  camacCcycle2(LAM_N,LAM_A,26,NULL,&stat);
# endif
# if defined(OUTREG)
  dc_outreg |= 0xff00; /* set   higher 8-bit */
  camacCcycle2(OUTREG,0,16,&dc_outreg,&stat);
# endif
}

/*
 *  Subsidiary function called at disabling interrupt
 */
static void dc_int_off() {
  int stat;
# if defined(OUTREG)
  dc_outreg &= 0x00ff; /* clear higher 8-bit */
  camacCcycle2(OUTREG,0,16,&dc_outreg,&stat);
# endif
# if (defined(LAM_N) && defined(LAM_A))
  camacCcycle2(LAM_N,LAM_A,24,NULL,&stat);
# endif
}

/*
 *  Main function of interrupt handler
 */
static void  dc_int(int irq, void *dev_id, struct pt_regs *regs) {
  int wp0,stat,i;
  if(dev_id!=dc_dev_id) return; /* interrupt from other devices sharing IRQ */
  DC_AUX_TSTL;                  /* ad hoc trap for buggy CC/7000AT-IF ! */
  dc_int_count++;
  if ((dc_wp-dc_rp) > EV_MAX-MAX_EVLEN) {
    /* Buffer is full */
    dc_int_lost++;
  } else {
    /* Data AcQuisition for 1-event */
    wp0=dc_wp++;                  /* save current pointer for event-length */
    /* User should modify from here */
    for (i=0;i<8;i++)             /* read 8 channels of TDC1 */
      camacCcycle2(TDC1,i,0,&dc_evbuff[(dc_wp++)%EV_MAX],&stat);
    /* ------------------ to   here */
    dc_evbuff[(dc_wp++)%EV_MAX]=DELIM;               /* store delimiter    */
    dc_evbuff[wp0%EV_MAX]=(short)((dc_wp-wp0)<<1);   /* store event length */
  }
  /* Also user should modify from here */
  camacCcycle2(TDC1,0,9,0,&stat);                          /* clear module */
# if defined(OUTREG)                        /* notify end-of-event-process */
  dc_outreg |= 0x00ff; /*  set  lower 8-bit */
  camacCcycle2(OUTREG,0,16,&dc_outreg ,&stat);
  dc_outreg &= 0xff00; /* clear lower 8-bit */
  camacCcycle2(OUTREG,0,16,&dc_outreg ,&stat);
# endif
  /* ----------------------- to   here */
  if ((dc_wp!=dc_rp) && (wakeups!=0)) wake_up(&wait_que);
}
